package cc.fascinated.bat.command;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.emoji.Emoji;

import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor
@Getter
public enum Category {
    GENERAL(Emoji.fromUnicode("U+2699"), "General"),
    FUN(Emoji.fromFormatted("U+1F973"), "Fun"),
    SERVER(Emoji.fromFormatted("U+1F5A5"), "Server"),
    MODERATION(Emoji.fromFormatted("U+1F6E0"), "Moderation"),
    UTILITY(Emoji.fromFormatted("U+1F6E0"), "Utility"),
    MEDIA(Emoji.fromFormatted("U+1F3A5"), "Media"),
    BEAT_SABER(Emoji.fromFormatted("U+1FA84"), "Beat Saber");

    /**
     * The emoji for the category
     */
    @NonNull private final Emoji emoji;

    /**
     * The name of the category
     */
    @NonNull private final String name;

    /**
     * Gets a category by its name
     *
     * @param name the name of the category
     * @return the category
     */
    public static Category getByName(String name) {
        for (Category category : Category.values()) {
            if (category.getName().equalsIgnoreCase(name)) {
                return category;
            }
        }
        return null;
    }

    /**
     * Gets the categories sorted by their name
     *
     * @return the sorted categories
     */
    public static List<Category> getSortedByName() {
        return Arrays.stream(Category.values()).sorted(Comparator.comparing(Category::getName)).toList();
    }
}
