package cc.fascinated.bat.features.base.commands.fun;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.Category;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(
        name = "8ball", description = "Ask the magic 8ball a question",
        guildOnly = false,
        userInstall = true,
        prefixAllowed = true,
        category = Category.FUN
)
public class EightBallCommand extends BatCommand {
    private final String[] responses = new String[]{
            "It is certain",
            "It is decidedly so",
            "Without a doubt",
            "Yes, definitely",
            "You may rely on it",
            "As I see it, yes",
            "Most likely",
            "Outlook good",
            "Yes",
            "Signs point to yes",
            "Reply hazy, try again",
            "Ask again later",
            "Better not tell you now",
            "Cannot predict now",
            "Concentrate and ask again",
            "Don't count on it",
            "My reply is no",
            "My sources say no",
            "Outlook not so good",
            "Very doubtful"
    };

    public EightBallCommand() {
        super.addOptions(new OptionData(OptionType.STRING, "question", "The question you want to ask the 8ball", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        String question = super.getArgument("question", arguments, 0, true, event).getAsString();
        if (question == null) {
            super.replyEmbed(commandMessage, event, EmbedUtils.errorEmbed()
                    .setDescription("You need to provide a question to ask the 8ball")
            );
            return;
        }

        String response = responses[(int) (Math.random() * responses.length)];
        super.replyEmbed(commandMessage, event, EmbedUtils.successEmbed()
                .setDescription("You asked: `%s`\n\n:8ball: The magic 8ball says: `%s`".formatted(question, response))
        );
    }
}
