package cc.fascinated.bat.features.base.commands.general;

import cc.fascinated.bat.Consts;
import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.Category;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.InteractionBuilder;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.CommandService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(
        name = "help",
        description = "View the bots command categories",
        userInstall = true,
        category = Category.GENERAL
)
public class HelpCommand extends BatCommand implements EventListener {
    private final CommandService commandService;

    @Autowired
    public HelpCommand(@NonNull CommandService commandService) {
        this.commandService = commandService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        InteractionBuilder interactionBuilder = new InteractionBuilder();
        interactionBuilder.addUrlButton("Invite Me", Consts.INVITE_URL, null);
        interactionBuilder.addUrlButton("Support Server", Consts.SUPPORT_INVITE_URL, null);
        interactionBuilder.addStringSelect("Home", "Return Home", Emojis.HOME_EMOJI, (buttonEvent) -> {
            buttonEvent.editMessageEmbeds(createHomeEmbed(event.isFromGuild())).queue();
        });
        for (Category category : Category.getSortedByName()) {
            List<BatCommand> categoryCommands = commandService.getCommandsByCategory(category, event.isFromGuild());
            if (categoryCommands.isEmpty()) {
                continue;
            }

            interactionBuilder.addStringSelect(
                    category.getName(),
                    "View commands in the %s category".formatted(category.getName()),
                    category.getEmoji(),
                    (buttonEvent) -> {
                        DescriptionBuilder description = new DescriptionBuilder(null);
                        description.appendLine("Commands in the **%s** Category".formatted(category.getName()), false);
                        description.emptyLine();

                        for (BatCommand command : categoryCommands) {
                            if (!command.getSubCommands().isEmpty()) {
                                for (BatCommand subCommand : command.getSubCommands().values()) {
                                    description.appendLine("`%s %s` - %s".formatted(
                                            command.getInfo().getName(),
                                            subCommand.getInfo().getName(),
                                            subCommand.getInfo().getDescription()
                                    ), true);
                                }
                                continue;
                            }
                            description.appendLine("`%s` - %s".formatted(
                                    command.getInfo().getName(),
                                    command.getInfo().getDescription()
                            ), true);
                        }

                        buttonEvent.editMessageEmbeds(EmbedUtils.genericEmbed()
                                .setDescription(description.build())
                                .build()).queue();
                    });
        }


        event.replyEmbeds(createHomeEmbed(event.isFromGuild())).addComponents(interactionBuilder.build()).queue();
    }

    /**
     * Creates the home embed for the help command
     *
     * @return The home embed
     */
    private MessageEmbed createHomeEmbed(boolean ranInsideGuild) {
        StringBuilder categories = new StringBuilder();
        for (Category category : Category.values()) {
            if (commandService.getCommandsByCategory(category, ranInsideGuild).isEmpty()) {
                continue;
            }
            long commandCount = commandService.getCommandsByCategory(category, ranInsideGuild).size();
            categories.append("➜  %s - **%s Command%s**\n".formatted(
                    category.getName(),
                    commandCount,
                    commandCount == 1 ? "" : "s"
            ));
        }

        return EmbedUtils.genericEmbed()
                .setDescription("""
                        **Welcome to the Bat Help Menu!**%s

                        %s
                        *View our [TOS](%s) and [Privacy Policy](%s) for more information.*
                        """.formatted(
                        !ranInsideGuild ? "\n*guild only commands won't be shown here*" : "",
                        categories.toString(),
                        Consts.TERMS_OF_SERVICE_URL,
                        Consts.PRIVACY_POLICY_URL
                ))
                .build();
    }
}
