package cc.fascinated.bat.features.base.commands.server;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.NumberFormatter;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

/**
 * @author Nick (okNick)
 */
@Component
@CommandInfo(name = "membercount", description = "View the member count of the server!")
public class MemberCountCommand extends BatCommand {
    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        Guild discordGuild = guild.getDiscordGuild();
        int totalMembers = 0, totalUsers = 0, totalBots = 0;
        for (Member guildMember : discordGuild.getMembers()) {
            if (guildMember.getUser().isBot()) {
                totalBots++;
            } else {
                totalUsers++;
            }
            totalMembers++;
        }
        event.replyEmbeds(EmbedUtils.genericEmbed()
                .setDescription("""
                **Member Count**
                Total Members: `%s`
                Total Users: `%s`
                Total Bots: `%s`
                """.formatted(
                        NumberFormatter.format(totalMembers),
                        NumberFormatter.format(totalUsers),
                        NumberFormatter.format(totalBots)
                )).build()).queue();
    }
}