package cc.fascinated.bat.features.base.commands.server.channel;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.Channel;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

import java.util.Objects;

/**
 * @author Nick (okNick)
 */
@Component
@CommandInfo(name = "settopic", description = "Set the topic of a channel", requiredPermissions = Permission.MANAGE_CHANNEL)
public class SetTopicSubCommand extends BatCommand {
    public SetTopicSubCommand() {
        super.addOptions(
                new OptionData(OptionType.STRING, "topic", "The topic to set", true),
                new OptionData(OptionType.CHANNEL, "channel", "The channel to set the topic of", false)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        Channel target = event.getOption("channel") == null ? channel : Objects.requireNonNull(event.getOption("channel")).getAsChannel();
        if (!(target instanceof TextChannel textChannel)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("<#%s> is not a text channel!".formatted(target.getId()))
                            .build())
                    .queue();
            return;
        }

        String topic = Objects.requireNonNull(event.getOption("topic")).getAsString();
        if (topic.length() > 1024) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("The topic must be 1024 characters or less!")
                            .build())
                    .queue();
            return;
        }
        event.deferReply().queue(hook -> textChannel.getManager().setTopic(topic).queue((voidd) -> hook.editOriginalEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully set the topic of <#%s> to: \"%s\"".formatted(textChannel.getId(), topic))
                .build()).queue()));
    }
}