package cc.fascinated.bat.features.birthday.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.birthday.UserBirthday;
import cc.fascinated.bat.features.birthday.profile.BirthdayProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * @author Fascinated (fascinated7)
 */
@Component("birthday:add.sub")
@CommandInfo(name = "set", description = "Add your birthday to this guild")
public class SetSubCommand extends BatCommand {
    private static final SimpleDateFormat FORMATTER = new SimpleDateFormat("dd/MM/yyyy");

    @Autowired
    public SetSubCommand() {
        super.addOptions(new OptionData(OptionType.STRING, "birthday", "Your birthday (format: DAY/MONTH/YEAR - 01/05/2004)", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        BirthdayProfile profile = guild.getBirthdayProfile();
        if (!profile.hasChannelSetup()) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Birthdays have not been enabled in this guild. Please ask an administrator to enable them.")
                    .build()).queue();
            return;
        }

        OptionMapping birthdayOption = event.getOption("birthday");
        assert birthdayOption != null;
        String birthdayString = birthdayOption.getAsString();

        Date birthday = parseBirthday(birthdayString);
        if (birthday == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("""
                            Invalid birthday format. Please use the following format:
                            DAY/MONTH/YEAR - 01/05/2004
                            """)
                    .build()).queue();
            return;
        }

        UserBirthday userBirthday = new UserBirthday();
        userBirthday.setBirthday(birthday);
        userBirthday.setHidden(false);
        profile.addBirthday(member.getId(), userBirthday);
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("You have updated your birthday!")
                .build()).queue();
    }

    /**
     * Parses a birthday from the string
     *
     * @param birthday the date to parse
     * @return the birthday
     */
    private Date parseBirthday(String birthday) {
        try {
            Date date = FORMATTER.parse(birthday);
            if (date.after(new Date())) {
                return null;
            }
            if (date.toInstant().toEpochMilli() < 0) {
                return null;
            }
            return date;
        } catch (ParseException ignored) {
            return null;
        }
    }
}
