package cc.fascinated.bat.features.counter.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.NumberFormatter;
import cc.fascinated.bat.features.counter.CounterChannel;
import cc.fascinated.bat.features.counter.CounterProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.ChannelType;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.entities.channel.unions.GuildChannelUnion;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("counter.remove:sub")
@CommandInfo(name = "remove", description = "Remove a counter channel")
public class RemoveSubCommand extends BatCommand {
    public RemoveSubCommand() {
        super.addOptions(new OptionData(OptionType.CHANNEL, "channel", "The channel to set the count in", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        OptionMapping channelOption = event.getOption("channel");
        assert channelOption != null;
        GuildChannelUnion targetChannel = channelOption.getAsChannel();
        if (targetChannel.getType() != ChannelType.TEXT) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s The channel must be a text channel".formatted(Emojis.CROSS_MARK_EMOJI))
                    .build()).queue();
            return;
        }
        TextChannel textChannel = targetChannel.asTextChannel();
        CounterProfile profile = guild.getCounterProfile();
        if (profile == null) {
            return;
        }

        // Check if the channel is a counter channel
        CounterChannel counterChannel = profile.getCounterChannel(textChannel.getId());
        if (counterChannel == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s The channel %s is not a counter channel".formatted(
                            Emojis.CROSS_MARK_EMOJI,
                            textChannel.getAsMention()
                    ))
                    .build()).queue();
            return;
        }

        profile.removeChannel(textChannel);
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("""
                        %s Successfully removed the counter channel %s
                        The count was `%s`
                        """.formatted(
                        Emojis.CHECK_MARK_EMOJI,
                        textChannel.getAsMention(),
                        NumberFormatter.simpleFormat(counterChannel.getCurrentCount())
                ))
                .build()).queue();
    }
}
