package cc.fascinated.bat.features.drag.listeners.request;

import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.drag.DragRequest;
import cc.fascinated.bat.features.drag.command.RequestSubCommand;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.interactions.InteractionHook;
import org.springframework.stereotype.Component;

import java.util.Optional;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class RequestListener implements EventListener {
    @Override
    public void onButtonInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ButtonInteractionEvent event) {
        if (guild == null) {
            return;
        }
        if (!event.getComponentId().equals("drag-request-cancel")) {
            return;
        }
        Optional<DragRequest> optionalDragRequest = RequestSubCommand.JOIN_REQUESTS.stream()
                .filter(request -> request.getMember().getId().equals(event.getUser().getId()))
                .findFirst();
        if (optionalDragRequest.isEmpty()) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s, you do not have a request to cancel.".formatted(user.getDiscordUser().getAsMention()))
                    .build()).queue();
            return;
        }
        DragRequest dragRequest = optionalDragRequest.get();
        InteractionHook interactionHook = dragRequest.getInteractionHook();
        interactionHook.editOriginalEmbeds(EmbedUtils.errorEmbed()
                .setDescription("You have cancelled your request to join %s's voice channel.".formatted(dragRequest.getTarget().getAsMention()))
                .build()).queue(message -> message.editMessageComponents().queue());
        dragRequest.getVoiceChannel().sendMessageEmbeds(EmbedUtils.errorEmbed()
                .setDescription("%s has cancelled their request to join your voice channel.".formatted(dragRequest.getMember().getAsMention()))
                .build()).queue();
        dragRequest.getRequestMessage().delete().queue();
        RequestSubCommand.JOIN_REQUESTS.remove(dragRequest);
    }
}
