package cc.fascinated.bat.features.leveling;

import cc.fascinated.bat.common.MathUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "Leveling Listener")
public class LevelingListener implements EventListener {
    private final LevelingFeature levelingFeature;

    @Autowired
    public LevelingListener(@NonNull LevelingFeature levelingFeature) {
        this.levelingFeature = levelingFeature;
    }

    @Override
    public void onGuildMessageReceive(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull MessageReceivedEvent event) {
        if (guild.getFeatureProfile().isFeatureDisabled(levelingFeature)) { // The feature is disabled
            return;
        }
        LevelingProfile profile = guild.getLevelingProfile();
        if (profile == null) {
            return;
        }

        UserLevel userLevel = profile.getUserLevel(user.getId());
        userLevel.addXp((int) MathUtils.random(15, 25));

        if (!userLevel.canLevelUp()) {
            return;
        }
        userLevel.levelup();

        TextChannel notificationChannel = profile.getNotificationChannel();
        if (notificationChannel == null) {
            return;
        }
        notificationChannel.sendMessage("Congratulations %s, you have leveled up to `%s`! :tada:".formatted(
                event.getAuthor().getAsMention(),
                userLevel.getLevel()
        )).queue();
    }
}
