package cc.fascinated.bat.features.logging.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.InteractionBuilder;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogCategory;
import cc.fascinated.bat.features.logging.LogProfile;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component("logs:list.sub")
@CommandInfo(name = "list", description = "See all the log types and their channels")
public class ListSubCommand extends BatCommand implements EventListener {
    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        LogProfile profile = guild.getLogProfile();

        InteractionBuilder interactionBuilder = new InteractionBuilder();
        interactionBuilder.addStringSelect("Home", "Return Home", Emojis.HOME_EMOJI, (buttonEvent) -> {
            buttonEvent.editMessageEmbeds(getHelpMessage()).queue();
        });

        for (LogCategory category : LogCategory.values()) {
            List<LogType> logEvents = LogType.getLogTypesByCategory(category.getName());
            DescriptionBuilder description = new DescriptionBuilder(null);
            description.appendLine("Log channels for the `%s` category".formatted(category.getName()), false);
            description.emptyLine();
            for (LogType logType : logEvents) {
                if (logType.getCategory() != category) {
                    continue;
                }
                TextChannel logChannel = profile.getLogChannel(logType);
                description.appendLine("%s: %s".formatted(logType.getName(), logChannel == null ? "Not Set" : logChannel.getAsMention()), true);
            }

            interactionBuilder.addStringSelect(
                    category.getName(),
                    "View log events in the %s category".formatted(category.getName()),
                    category.getEmoji(),
                    (buttonEvent) -> buttonEvent.editMessageEmbeds(EmbedUtils.genericEmbed().setDescription(description.build()).build()).queue()
            );
        }


        event.replyEmbeds(getHelpMessage()).addComponents(interactionBuilder.build()).queue();
    }

    /**
     * Gets the help message for the command
     *
     * @return the help message
     */
    public MessageEmbed getHelpMessage() {
        return EmbedUtils.genericEmbed().setDescription(new DescriptionBuilder(null).appendLine(
                """
                        Set the log channel for:
                        - A specific event, use `/logs set <event> <channel>`
                        - A specific category by using `/logs set <category> <channel>`
                        - All log events by using `/logs set all <channel>`
                        To remove a log channel, it's the same as setting it,
                        but with `/logs remove` instead of `/logs set`
                        \s
                        *Use the menu below to see the log events for each category*
                        """, false).build()).build();
    }
}

