package cc.fascinated.bat.features.logging.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.logging.LogCategory;
import cc.fascinated.bat.features.logging.LogProfile;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component("logs:set.sub")
@CommandInfo(name = "set", description = "Set the channel for a log type")
public class SetSubCommand extends BatCommand {
    public SetSubCommand() {
        super.addOptions(
                new OptionData(OptionType.STRING, "type", "The type of log to set", true),
                new OptionData(OptionType.CHANNEL, "channel", "The channel to set the log to", true)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        OptionMapping typeOption = event.getOption("type");
        assert typeOption != null;
        OptionMapping channelOption = event.getOption("channel");
        assert channelOption != null;

        String type = typeOption.getAsString();
        TextChannel targetChannel = channelOption.getAsChannel().asTextChannel();
        LogProfile profile = guild.getLogProfile();

        // Set the log channel for all log types
        if (type.equalsIgnoreCase("all")) {
            for (LogType logType : LogType.values()) {
                profile.setLogChannel(logType, targetChannel);
            }
            DescriptionBuilder description = new DescriptionBuilder(null);
            description.appendLine("%s Successfully set the log channel for `%s` log events to %s".formatted(
                    Emojis.CHECK_MARK_EMOJI,
                    LogType.values().length,
                    targetChannel.getAsMention()
            ), false);
            description.emptyLine();
            description.appendLine("*To view the current log channels, use `/logs list`*", false);
            event.replyEmbeds(EmbedUtils.successEmbed().setDescription(description.build()).build()).queue();
            return;
        }

        // Set the log channel for a specific log category
        LogCategory logCategory = LogCategory.getLogCategory(type);
        if (logCategory != null) {
            List<LogType> category = LogType.getLogTypesByCategory(type);
            for (LogType logType : category) {
                profile.setLogChannel(logType, targetChannel);
            }
            DescriptionBuilder description = new DescriptionBuilder(null);
            description.appendLine("%s Successfully set the log channel for `%s` `%s` log events to %s".formatted(
                    Emojis.CHECK_MARK_EMOJI,
                    category.size(),
                    logCategory.getName(),
                    targetChannel.getAsMention()
            ), false);
            description.emptyLine();
            description.appendLine("*To view the current log channels, use `/logs list`*", false);
            event.replyEmbeds(EmbedUtils.successEmbed()
                    .setDescription(description.build())
                    .build()).queue();
            return;
        }

        // Set the log channel for a specific log type
        LogType logType = LogType.getLogType(type);
        if (logType == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s Invalid log type".formatted(Emojis.CROSS_MARK_EMOJI))
                    .build()).queue();
            return;
        }
        profile.setLogChannel(logType, targetChannel);
        event.replyEmbeds(EmbedUtils.successEmbed().setDescription(new DescriptionBuilder(null)
                        .appendLine("%s Successfully set the log channel for `%s` to %s".formatted(
                                Emojis.CHECK_MARK_EMOJI,
                                logType.getName(),
                                targetChannel.getAsMention()
                        ), false)
                        .emptyLine()
                        .appendLine("*To view the current log channels, use `/logs list`*", false)
                        .build()).build()).queue();
    }
}
