package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.emoji.Emoji;
import net.dv8tion.jda.api.events.emoji.EmojiAddedEvent;
import net.dv8tion.jda.api.events.emoji.EmojiRemovedEvent;
import net.dv8tion.jda.api.events.emoji.update.EmojiUpdateNameEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "Logging - Emoji Listener")
public class EmojiListener implements EventListener {
    private final LogFeature logFeature;

    @Autowired
    public EmojiListener(@NonNull LogFeature logFeature) {
        this.logFeature = logFeature;
    }

    @Override
    public void onEmojiAdd(@NonNull BatGuild guild, @NonNull Emoji emoji, @NonNull EmojiAddedEvent event) {
        log.info("Emoji \"{}\" was added in guild \"{}\"", emoji.getName(), guild.getName());
        logFeature.sendLog(guild, LogType.EMOJI_ADD, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Emoji Added")
                        .appendLine("Emoji: %s".formatted(emoji.getFormatted()), true)
                        .appendLine("Name: `%s`".formatted(emoji.getName()), true)
                        .build())
                .build());
    }

    @Override
    public void onEmojiRemove(@NonNull BatGuild guild, @NonNull Emoji emoji, @NonNull EmojiRemovedEvent event) {
        log.info("Emoji \"{}\" was removed in guild \"{}\"", emoji.getName(), guild.getName());
        logFeature.sendLog(guild, LogType.EMOJI_REMOVE, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Emoji Removed")
                        .appendLine("Emoji: %s".formatted(emoji.getFormatted()), true)
                        .appendLine("Name: `%s`".formatted(emoji.getName()), true)
                        .build())
                .build());
    }

    @Override
    public void onEmojiRename(@NonNull BatGuild guild, @NonNull Emoji emoji, @NonNull String oldName, @NonNull String newName, @NonNull EmojiUpdateNameEvent event) {
        log.info("Emoji \"{}\" was renamed to \"{}\" in guild \"{}\"", oldName, newName, guild.getName());
        logFeature.sendLog(guild, LogType.EMOJI_NAME_UPDATED, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Emoji Renamed")
                        .appendLine("Emoji: %s".formatted(emoji.getFormatted()), true)
                        .appendLine("Name: `%s` -> `%s`".formatted(oldName, newName), true)
                        .build())
                .build());
    }
}
