package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.DiscordMessage;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.events.message.MessageDeleteEvent;
import net.dv8tion.jda.api.events.message.MessageUpdateEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "Logging - Message Listener")
public class MessageListener implements EventListener {
    private final LogFeature logFeature;

    @Autowired
    public MessageListener(@NonNull ApplicationContext context) {
        this.logFeature = context.getBean(LogFeature.class);
    }

    @Override
    public void onGuildMessageDelete(@NonNull BatGuild guild, BatUser user, DiscordMessage message, @NonNull MessageDeleteEvent event) {
        if (user == null || user.getDiscordUser().isBot() || message.getAuthor().isBot()) return;
        log.info("User \"{}\" deleted a message in guild \"{}\"", user.getDiscordUser().getGlobalName(), guild.getName());

        logFeature.sendLog(guild, LogType.MESSAGE_DELETE, EmbedUtils.errorEmbed()
                .setDescription(new DescriptionBuilder("Message Deleted")
                        .appendLine("Author: %s".formatted(message.getAuthor().getAsMention()), true)
                        .appendLine("Channel: %s".formatted(message.getChannel().getAsMention()), true)
                        .appendLine("Content: %s".formatted(logFeature.formatContent(message.getContent())), true)
                        .build())
                .build());
    }

    @Override
    public void onGuildMessageEdit(@NonNull BatGuild guild, @NonNull BatUser user, DiscordMessage oldMessage,
                                   @NonNull DiscordMessage newMessage, @NonNull MessageUpdateEvent event) {
        if (user.getDiscordUser().isBot() || newMessage.getAuthor().isBot() || oldMessage == null) return;
        log.info("User \"{}\" edited a message in guild \"{}\"", user.getDiscordUser().getGlobalName(), guild.getName());

        logFeature.sendLog(guild, LogType.MESSAGE_EDIT, EmbedUtils.genericEmbed()
                .setDescription(new DescriptionBuilder("Message Edited")
                        .appendLine("Author: %s".formatted(newMessage.getAuthor().getAsMention()), true)
                        .appendLine("Channel: %s".formatted(newMessage.getChannel().getAsMention()), true)
                        .appendLine("Old Content: %s".formatted(logFeature.formatContent(oldMessage.getContent())), true)
                        .appendLine("New Content: %s".formatted(logFeature.formatContent(newMessage.getContent())), true)
                        .appendLine("*[Jump to Message](%s)*".formatted(newMessage.getMessageUrl()), false)
                        .build())
                .build());
    }
}
