package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.sticker.GuildSticker;
import net.dv8tion.jda.api.events.sticker.GuildStickerAddedEvent;
import net.dv8tion.jda.api.events.sticker.GuildStickerRemovedEvent;
import net.dv8tion.jda.api.events.sticker.update.GuildStickerUpdateNameEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "Logging - Sticker Listener")
public class StickerListener implements EventListener {
    private final LogFeature logFeature;

    @Autowired
    public StickerListener(@NonNull LogFeature logFeature) {
        this.logFeature = logFeature;
    }

    @Override
    public void onGuildStickerAdd(@NonNull BatGuild guild, @NonNull GuildSticker sticker, @NonNull GuildStickerAddedEvent event) {
        log.info("Sticker \"{}\" was added in guild \"{}\"", sticker.getName(), guild.getName());
        logFeature.sendLog(guild, LogType.STICKER_ADD, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Sticker Added")
                        .appendLine("Sticker: %s".formatted(sticker.getName()), true)
                        .appendLine("URL: %s".formatted(sticker.getIconUrl()), true)
                        .build())
                .build());
    }

    @Override
    public void onGuildStickerRemove(@NonNull BatGuild guild, @NonNull GuildSticker sticker, @NonNull GuildStickerRemovedEvent event) {
        log.info("Sticker \"{}\" was removed in guild \"{}\"", sticker.getName(), guild.getName());
        logFeature.sendLog(guild, LogType.STICKER_REMOVE, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Sticker Removed")
                        .appendLine("Sticker: %s".formatted(sticker.getName()), true)
                        .appendLine("URL: %s".formatted(sticker.getIconUrl()), true)
                        .build())
                .build());
    }

    @Override
    public void onGuildStickerRename(@NonNull BatGuild guild, @NonNull GuildSticker sticker, @NonNull String oldName,
                                     @NonNull String newName, @NonNull GuildStickerUpdateNameEvent event) {
        log.info("Sticker \"{}\" was renamed in guild \"{}\" from \"{}\" to \"{}\"", sticker.getName(), guild.getName(), oldName, newName);
        logFeature.sendLog(guild, LogType.STICKER_NAME_UPDATED, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Sticker Renamed")
                        .appendLine("Sticker: %s".formatted(sticker.getName()), true)
                        .appendLine("Name: `%s` -> `%s`".formatted(oldName, newName), true)
                        .build())
                .build());
    }
}
