package cc.fascinated.bat.features.minecraft.command.minecraft;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;
import xyz.mcutils.McUtilsAPI;
import xyz.mcutils.models.cache.CachedPlayer;
import xyz.mcutils.models.player.Skin;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(
        name = "lookup-player",
        description = "Lookup a Minecraft player"
)
public class LookupPlayerSubCommand extends BatCommand {
    public LookupPlayerSubCommand() {
        super.addOptions(
                new OptionData(OptionType.STRING, "player", "The player to lookup", true)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        OptionMapping playerOption = event.getOption("player");
        assert playerOption != null;

        String player = playerOption.getAsString();
        CachedPlayer cachedPlayer = McUtilsAPI.getPlayer(player);
        if (cachedPlayer == null) {
            event.reply("The player `%s` could not be found".formatted(player)).queue();
            return;
        }

        String headUrl = cachedPlayer.getSkin() != null ? cachedPlayer.getSkin().getParts().get(Skin.SkinPart.HEAD.getName()) : null;
        DescriptionBuilder description = new DescriptionBuilder("Player Lookup")
                .appendLine("Username: `%s`".formatted(cachedPlayer.getUsername()), true)
                .appendLine("UUID: `%s`".formatted(cachedPlayer.getUniqueId().toString()), true);
        if (cachedPlayer.getSkin() != null) {
            description.appendLine("Skin: [Click Here](%s)".formatted(headUrl), true);
        }
        if (cachedPlayer.getCape() != null) {
            description.appendLine("Cape: [Click Here](%s)".formatted(cachedPlayer.getCape().getUrl()), true);
        }
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription(description.build())
                .setThumbnail(headUrl)
                .build()).queue();
    }
}
