package cc.fascinated.bat.features.moderation.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.TimeUtils;
import cc.fascinated.bat.features.moderation.punish.PunishmentProfile;
import cc.fascinated.bat.features.moderation.punish.PunishmentType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(
        name = "ban",
        description = "Bans a member",
        requiredPermissions = Permission.BAN_MEMBERS
)
public class BanCommand extends BatCommand {
    private final UserService userService;

    @Autowired
    public BanCommand(@NonNull UserService userService) {
        this.userService = userService;
        super.addOptions(
                new OptionData(OptionType.USER, "member", "The member you want to ban", true),
                new OptionData(OptionType.STRING, "reason", "The reason for the ban", false),
                new OptionData(OptionType.STRING, "length", "The length of the ban", false)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        OptionMapping memberOption = event.getOption("member");
        OptionMapping reasonOption = event.getOption("reason");
        OptionMapping lengthOption = event.getOption("length");
        assert memberOption != null;
        BatUser targetUser = userService.getUser(memberOption.getAsUser().getId());
        if (targetUser == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("User not found")
                    .build()
            ).queue();
            return;
        }
        String reason = reasonOption == null ? null : reasonOption.getAsString();
        long length = lengthOption == null ? -1 : TimeUtils.fromString(lengthOption.getAsString());

        if (lengthOption != null && length == -1) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Invalid length provided")
                    .build()
            ).queue();
            return;
        }

        PunishmentProfile profile = guild.getPunishmentProfile();
        if (!profile.canPunish(guild, user, targetUser)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You cannot ban this user")
                    .build()
            ).queue();
            return;
        }

        profile.temporaryPunish(targetUser, user, guild, PunishmentType.BAN, reason, length);
        profile.punishmentResponse(event, PunishmentType.BAN, targetUser, reason, false, length);
    }
}
