package cc.fascinated.bat.features.moderation.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.moderation.punish.Punishment;
import cc.fascinated.bat.features.moderation.punish.PunishmentProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(
        name = "punish-history",
        description = "View the punishment history of a member",
        requiredPermissions = Permission.MANAGE_SERVER
)
public class PunishHistoryCommand extends BatCommand {
    private static final int PUNISHMENTS_PER_PAGE = 10;
    private final UserService userService;

    @Autowired
    public PunishHistoryCommand(@NonNull UserService userService) {
        this.userService = userService;
        super.addOptions(
                new OptionData(OptionType.USER, "member", "The member you want to view the punishment history of", true)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        OptionMapping memberOption = event.getOption("member");
        assert memberOption != null;
        BatUser targetUser = userService.getUser(memberOption.getAsUser().getId());
        if (targetUser == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("User not found")
                    .build()
            ).queue();
            return;
        }

        PunishmentProfile profile = guild.getPunishmentProfile();
        List<Punishment> punishments = profile.getPunishments(targetUser);

        // todo: pagination
        DescriptionBuilder history = new DescriptionBuilder("Punishment history for %s".formatted(targetUser.getDiscordUser().getAsMention()));
        if (!punishments.isEmpty()) {
            for (int i = 0; i < Math.min(punishments.size(), 10); i++) {
                Punishment punishment = punishments.get(i);
                BatUser issuer = userService.getUser(punishment.getIssuer());
                history.appendLine("> **[%s%s]** <t:%s:R> Reason: `%s` | Issuer: %s%s".formatted(
                        punishment.getEndDate() != null ? "TEMP-" : "",
                        punishment.getType().name(),
                        punishment.getIssuedDate().toInstant().getEpochSecond(),
                        punishment.getReason() == null ? "N/A" : punishment.getReason(),
                        issuer.getDiscordUser().getAsMention(),
                        punishment.isExpired() ? " **(Expired)**" : ""
                ), false);
            }
        } else {
            history.appendLine("No punishments found", false);
        }

        event.replyEmbeds(EmbedUtils.genericEmbed()
                .setDescription(history.build())
                .build()
        ).queue();
    }
}
