package cc.fascinated.bat.features.namehistory.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.namehistory.TrackedName;
import cc.fascinated.bat.features.namehistory.profile.user.NameHistoryProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("namehistory:user.sub")
@CommandInfo(name = "user", description = "View the global name history of a user", guildOnly = false)
public class UserSubCommand extends BatCommand {
    private final UserService userService;

    @Autowired
    public UserSubCommand(@NonNull UserService userService) {
        this.userService = userService;
        super.addOptions(new OptionData(OptionType.USER, "user", "The user to view the name history of", false));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        OptionMapping userOption = event.getOption("user");
        BatUser target = userOption == null ? user : userService.getUser(userOption.getAsUser().getId());
        if (target == null) {
            channel.sendMessage("User not found").queue();
            return;
        }

        NameHistoryProfile profile = target.getNameHistoryProfile();
        StringBuilder builder = new StringBuilder();
        if (profile.getNameHistory().isEmpty()) {
            builder.append("%s has no name history".formatted(target.getDiscordUser().getAsMention()));
        } else {
            for (TrackedName trackedName : profile.getNameHistorySorted()) {
                builder.append("`%s` - <t:%s>\n".formatted(trackedName.getName(), trackedName.getChangedDate().toInstant().getEpochSecond()));
            }
        }

        event.replyEmbeds(EmbedUtils.genericEmbed()
                .setAuthor("%s's Global Name History".formatted(target.getName()))
                .setDescription(builder.toString())
                .build()
        ).queue();
    }
}
