package cc.fascinated.bat.features.scoresaber.command.scoresaber;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.Category;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.*;
import cc.fascinated.bat.features.scoresaber.profile.user.ScoreSaberProfile;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberAccountToken;
import cc.fascinated.bat.service.ScoreSaberService;
import lombok.NonNull;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "scoresaber", description = "General ScoreSaber commands", userInstall = true, category = Category.BEAT_SABER)
public class ScoreSaberCommand extends BatCommand {
    @Autowired
    public ScoreSaberCommand(@NonNull ApplicationContext context) {
        super.addSubCommands(
                context.getBean(LinkSubCommand.class),
                context.getBean(UserSubCommand.class),
                context.getBean(MeSubCommand.class),
                context.getBean(ResetSubCommand.class),
                context.getBean(ScoresSummarySubCommand.class)
        );
    }

    /**
     * Builds the profile embed for the ScoreSaber profile
     *
     * @param user              The user to build the profile embed for
     * @param scoreSaberService The ScoreSaber service
     * @param event             The interaction
     */
    public static void sendProfileEmbed(boolean isSelf, BatUser user, ScoreSaberService scoreSaberService, SlashCommandInteraction event) {
        ScoreSaberProfile profile = user.getScoreSaberProfile();
        if (profile.getAccountId() == null) {
            if (!isSelf) {
                event.replyEmbeds(EmbedUtils.errorEmbed()
                                .setDescription("%s does not have a linked ScoreSaber account".formatted(user.getDiscordUser().getAsMention()))
                                .build())
                        .setEphemeral(true)
                        .queue();
            }
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You do not have a linked ScoreSaber account")
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        long before = System.currentTimeMillis();
        ScoreSaberAccountToken account = scoreSaberService.getAccount(profile.getAccountId());
        if (account == null) {
            if (!isSelf) {
                event.replyEmbeds(EmbedUtils.errorEmbed()
                                .setDescription("%s has an invalid ScoreSaber account linked, please ask them to re-link their account"
                                        .formatted(user.getDiscordUser().getAsMention()))
                                .build())
                        .setEphemeral(true)
                        .queue();
            }
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You have an invalid ScoreSaber account linked, please re-link your account")
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        if (profile.getAccountId() == null) {
            if (!isSelf) {
                event.replyEmbeds(EmbedUtils.errorEmbed()
                                .setDescription("%s does not have a linked ScoreSaber account".formatted(user.getDiscordUser().getAsMention()))
                                .build())
                        .setEphemeral(true)
                        .queue();
            }
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You do not have a linked ScoreSaber account")
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        event.replyEmbeds(EmbedUtils.genericEmbed()
                .setDescription("Loading profile for %s...".formatted(user.getDiscordUser().getAsMention()))
                .build()).queue(message -> {
            ScoreSaberService.RawPerGlobal rawPerGlobal = scoreSaberService.getRawPerGlobal(profile, (callback) -> {
                int currentPage = callback.getCurrentPage();
                int totalPages = callback.getTotalPages();
                // Only update every 5 pages, but show the first page
                if (currentPage % 5 != 0 && currentPage != 1) {
                    return;
                }
                message.editOriginalEmbeds(EmbedUtils.genericEmbed()
                        .setDescription("""
                                    Loading profile for %s...
                                    Page `%s`/`%s`
                                    """.formatted(
                                user.getDiscordUser().getAsMention(),
                                currentPage,
                                totalPages
                        ))
                        .build()).queue();
            });

            String name = account.getName();
            String country = account.getCountry();
            String rank = NumberFormatter.simpleFormat(account.getRank());
            String countryRank = NumberFormatter.simpleFormat(account.getCountryRank());
            String pp = NumberFormatter.simpleFormat(account.getPp());
            String ppPerRawGlobal = NumberFormatter.simpleFormat(rawPerGlobal.getRawPerGlobal());
            long joinedTimeInSeconds = DateUtils.getDateFromString(account.getFirstSeen()).toInstant().getEpochSecond();
            String timeTaken = TimeUtils.format(System.currentTimeMillis() - before, TimeUtils.BatTimeFormat.FIT, true);

            message.editOriginalEmbeds(EmbedUtils.successEmbed()
                    .setDescription(new DescriptionBuilder("%s's ScoreSaber Account".formatted(user.getDiscordUser().getAsMention()))
                            .appendLine("Name: `%s`".formatted(name), true)
                            .appendLine("Country: `%s`".formatted(country), true)
                            .appendLine("Rank: `#%s`".formatted(rank), true)
                            .appendLine("Country Rank: `#%s`".formatted(countryRank), true)
                            .appendLine("PP: `%spp`".formatted(pp), true)
                            .appendLine("Raw PP Per Global: `%spp`".formatted(ppPerRawGlobal), true)
                            .appendLine("Joined: <t:%s>".formatted(joinedTimeInSeconds), true)
                            .build())
                    .setThumbnail(account.getProfilePicture())
                    .setFooter("Took %s (%s/%s cached pages)".formatted(
                            timeTaken,
                            rawPerGlobal.getCachedPages(),
                            rawPerGlobal.getTotalPages()
                    ), null)
                    .build()).queue();
        });
    }
}
