package cc.fascinated.bat.features.scoresaber.command.scoresaber;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.NumberFormatter;
import cc.fascinated.bat.features.scoresaber.profile.user.ScoreSaberProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberLeaderboardToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberPlayerScoreToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberScoreToken;
import cc.fascinated.bat.service.ScoreSaberService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.utils.FileUpload;
import net.steppschuh.markdowngenerator.table.Table;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component("scoresaber:scores-summary.sub")
@CommandInfo(name = "scores-summary", description = "Generate a summary of your scores")
public class ScoresSummarySubCommand extends BatCommand {
    private final ScoreSaberService scoreSaberService;

    @Autowired
    public ScoresSummarySubCommand(@NonNull ScoreSaberService scoreSaberService) {
        this.scoreSaberService = scoreSaberService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        ScoreSaberProfile profile = user.getScoreSaberProfile();

        event.replyEmbeds(EmbedUtils.genericEmbed()
                        .setDescription("Loading profile for %s...".formatted(user.getDiscordUser().getAsMention()))
                        .build())
                .queue(message -> {
                    List<ScoreSaberService.CachedPage> pages = scoreSaberService.getScores(profile, (currentPage -> {
                        // Only update every 5 pages, but show the first page
                        if (currentPage.getCurrentPage() % 5 != 0 && currentPage.getCurrentPage() != 1) {
                            return;
                        }

                        message.editOriginalEmbeds(EmbedUtils.genericEmbed()
                                .setDescription("Loading profile for %s... (Page %s/%s)".formatted(
                                        user.getDiscordUser().getAsMention(),
                                        currentPage.getCurrentPage(),
                                        currentPage.getTotalPages()
                                ))
                                .build()).queue();
                    }));

                    Table.Builder tableBuilder = new Table.Builder()
                            .withAlignments(Table.ALIGN_LEFT, Table.ALIGN_LEFT, Table.ALIGN_LEFT, Table.ALIGN_LEFT)
                            .addRow("Song", "PP", "Accuracy", "Rank");

                    int totalRankedScores = 0;
                    List<ScoreSaberPlayerScoreToken> scores = new ArrayList<>();
                    for (ScoreSaberService.CachedPage page : pages) {
                        Collections.addAll(scores, page.getPage().getPlayerScores());
                    }

                    // Sort by highest PP first
                    scores.sort((score1, score2) -> {
                        if (score1.getScore().getPp() > score2.getScore().getPp()) {
                            return -1;
                        } else if (score1.getScore().getPp() < score2.getScore().getPp()) {
                            return 1;
                        }
                        return 0;
                    });

                    // Add the scores to the table
                    for (ScoreSaberPlayerScoreToken scoreToken : scores) {
                        ScoreSaberScoreToken score = scoreToken.getScore();
                        ScoreSaberLeaderboardToken leaderboard = scoreToken.getLeaderboard();
                        double acc = leaderboard.getMaxScore() == 0 ? 0 : ((double) score.getBaseScore() / leaderboard.getMaxScore()) * 100;

                        tableBuilder.addRow(
                                "[%s](https://scoresaber.com/leaderboard/%s)".formatted(
                                        leaderboard.getSongName(),
                                        leaderboard.getId()
                                ),
                                NumberFormatter.simpleFormat(score.getPp()) + "pp",
                                NumberFormatter.simpleFormat(acc) + "%",
                                "#" + NumberFormatter.simpleFormat(score.getRank())
                        );

                        if (score.getPp() != 0) {
                            totalRankedScores++;
                        }
                    }

                    message.editOriginalEmbeds(EmbedUtils.genericEmbed()
                                    .setDescription("""
                                            **Scores Summary**
                                            Here is a summary of score for %s
                                            \s
                                            Total Scores: %s
                                            Total Ranked Scores: %s
                                            """.formatted(
                                            user.getDiscordUser().getAsMention(),
                                            NumberFormatter.simpleFormat(scores.size()),
                                            NumberFormatter.simpleFormat(totalRankedScores)
                                    ))
                                    .build())
                            .queue();
                    channel.sendFiles(FileUpload.fromData(tableBuilder.build().toString().getBytes(), "scores-summary.md")).queue();
                });
    }
}
