package cc.fascinated.bat.features.scoresaber.command.scoresaber;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.ScoreSaberService;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("scoresaber:user.sub")
@CommandInfo(name = "user", description = "Gets a ScoreSaber profile")
public class UserSubCommand extends BatCommand {
    private final ScoreSaberService scoreSaberService;
    private final UserService userService;

    @Autowired
    public UserSubCommand(@NonNull ScoreSaberService scoreSaberService, @NonNull UserService userService) {
        super.addOptions(new OptionData(OptionType.USER, "user", "The user to view the ScoreSaber profile of", true));
        this.scoreSaberService = scoreSaberService;
        this.userService = userService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        OptionMapping userOption = event.getOption("user");
        assert userOption != null;

        BatUser target = userService.getUser(userOption.getAsUser().getId());
        if (target == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("Unknown user")
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }
        ScoreSaberCommand.sendProfileEmbed(false, target, scoreSaberService, event);
    }
}
