package cc.fascinated.bat.features.spotify.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.SpotifyService;
import lombok.NonNull;
import lombok.SneakyThrows;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.events.interaction.ModalInteractionEvent;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.components.ActionRow;
import net.dv8tion.jda.api.interactions.components.buttons.Button;
import net.dv8tion.jda.api.interactions.components.text.TextInput;
import net.dv8tion.jda.api.interactions.components.text.TextInputStyle;
import net.dv8tion.jda.api.interactions.modals.Modal;
import net.dv8tion.jda.api.interactions.modals.ModalMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "link", description = "Link your Spotify account")
public class LinkSubCommand extends BatCommand implements EventListener {
    private final SpotifyService spotifyService;

    @Autowired
    public LinkSubCommand(@NonNull SpotifyService spotifyService) {
        this.spotifyService = spotifyService;
    }

    @Override @SneakyThrows
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
//        if (!user.getId().equals(Consts.BOT_OWNER)) {
//            event.replyEmbeds(EmbedUtils.errorEmbed()
//                            .setDescription("""
//                            %s We are currently awaiting Spotify's approval for our application. Please check back later.
//                            Submitted on: <t:1719583353>
//                            """.formatted(Emojis.CROSS_MARK_EMOJI))
//                            .build())
//                    .setEphemeral(true)
//                    .queue();
//            return;
//        }
        event.replyEmbeds(EmbedUtils.genericEmbed()
                        .setDescription("%s You can link your Spotify account by clicking [here](%s)".formatted(
                                Emojis.SPOTIFY_EMOJI.getFormatted(),
                                spotifyService.getAuthorizationUrl()
                        )).build())
                .addComponents(ActionRow.of(Button.primary("spotify_link", "Link Code").withEmoji(Emojis.SPOTIFY_EMOJI)))
                .setEphemeral(true)
                .queue();
    }

    @Override
    public void onButtonInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ButtonInteractionEvent event) {
        if (!event.getComponentId().equals("spotify_link")) {
            return;
        }

        TextInput code = TextInput.create("code", "Link Code", TextInputStyle.SHORT)
                .setPlaceholder("Your link code")
                .setMinLength(0)
                .setMaxLength(16)
                .build();

        Modal modal = Modal.create("link_modal", "Link Spotify Account")
                .addComponents(ActionRow.of(code))
                .build();

        event.replyModal(modal).queue();
    }

    @Override @SneakyThrows
    public void onModalInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ModalInteractionEvent event) {
        if (!event.getModalId().equals("link_modal")) {
            return;
        }

        ModalMapping codeMapping = event.getValue("code");
        if (codeMapping == null) {
            return;
        }
        String code = codeMapping.getAsString();
        if (!spotifyService.isValidLinkCode(code)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("%s The link code you provided is invalid.".formatted(Emojis.CROSS_MARK_EMOJI))
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        spotifyService.linkAccount(user, code);
        event.replyEmbeds(EmbedUtils.successEmbed()
                        .setDescription("%s You have linked your Spotify account!".formatted(Emojis.CHECK_MARK_EMOJI.getFormatted()))
                        .build())
                .setEphemeral(true)
                .queue();
    }
}
