package cc.fascinated.bat.features.statschannel;

import cc.fascinated.bat.common.NumberFormatter;
import cc.fascinated.bat.model.BatGuild;
import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor
@Getter
public enum StatsPlaceholders {
    MEMBER_COUNT("{member_count}", BatGuild.class) {
        @Override
        public String replacePlaceholder(Object object) {
            return NumberFormatter.simpleFormat(((BatGuild) object).getDiscordGuild().getMembers().size());
        }
    },
    BOT_COUNT("{bot_count}", BatGuild.class) {
        @Override
        public String replacePlaceholder(Object object) {
            return NumberFormatter.simpleFormat(((BatGuild) object).getDiscordGuild().getMembers().stream()
                    .filter(member -> member.getUser().isBot()).count());
        }
    },
    CHANNEL_COUNT("{channel_count}", BatGuild.class) {
        @Override
        public String replacePlaceholder(Object object) {
            return NumberFormatter.simpleFormat(((BatGuild) object).getDiscordGuild().getChannels().size());
        }
    },
    ROLE_COUNT("{role_count}", BatGuild.class) {
        @Override
        public String replacePlaceholder(Object object) {
            return NumberFormatter.simpleFormat(((BatGuild) object).getDiscordGuild().getRoles().size());
        }
    };

    /**
     * The placeholder string that will get replaced
     */
    private final String placeholder;

    /**
     * The class that the placeholder is associated with
     */
    private final Class<?> clazz;

    /**
     * Replaces the placeholder with the string based on the overridden method
     *
     * @param object The object to replace the placeholder with
     * @return The string with the placeholder replaced
     */
    public String replacePlaceholder(Object object) {
        if (clazz != null && !clazz.isInstance(object)) {
            throw new IllegalArgumentException("Object is not an instance of " + clazz.getName());
        }
        return null;
    }

    /**
     * Replaces all placeholders in a message with the objects provided
     *
     * @param message The message to replace the placeholders in
     * @param objects The objects to replace the placeholders with
     * @return The message with the placeholders replaced
     */
    public static String replaceAllPlaceholders(String message, Object... objects) {
        for (StatsPlaceholders placeholder : values()) {
            for (Object object : objects) {
                if (placeholder.getClazz() != null && !placeholder.getClazz().isInstance(object)) {
                    continue;
                }
                message = message.replace(placeholder.getPlaceholder(), placeholder.replacePlaceholder(object));
            }
        }
        return message;
    }
}
