package cc.fascinated.bat.features.statschannel.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.statschannel.StatsChannel;
import cc.fascinated.bat.features.statschannel.StatsChannelFeature;
import cc.fascinated.bat.features.statschannel.StatsChannelProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.concrete.VoiceChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
@Component("stats-channel.current:sub")
@CommandInfo(
        name = "current",
        description = "Shows the current stats channels"
)
public class CurrentSubCommand extends BatCommand {
    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        StatsChannelProfile profile = guild.getStatsChannelProfile();
        DescriptionBuilder builder = new DescriptionBuilder("Stats Channels");
        if (profile.getChannels().isEmpty()) {
            builder.appendLine("You have no stats channels set up", true);
        } else {
            for (Map.Entry<VoiceChannel, StatsChannel> entry : profile.getChannels().entrySet()) {
                builder.appendLine("%s: `%s`".formatted(entry.getKey().getAsMention(), entry.getValue().getDisplay()), true);
            }
        }
        builder.emptyLine();
        builder.appendLine(StatsChannelFeature.getPlaceholders(guild), false);

        event.replyEmbeds(EmbedUtils.successEmbed()
                        .setDescription(builder.build())
                        .build())
                .queue();
    }
}
