package cc.fascinated.bat.features.welcomer.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.HexColorUtils;
import cc.fascinated.bat.features.welcomer.WelcomerProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("welcomer:embed.sub")
@CommandInfo(name = "embed", description = "Set the welcomer embed (this will remove the welcomer plain message if set)")
public class EmbedSubCommand extends BatCommand {
    public EmbedSubCommand() {
        super.addOptions(
                new OptionData(OptionType.BOOLEAN, "ping-before-send", "Should we ping the user before sending the message?", true),
                new OptionData(OptionType.STRING, "description", "The description of the embed", true),
                new OptionData(OptionType.STRING, "color", "The color of the embed", true),
                new OptionData(OptionType.STRING, "title", "The title of the embed (only set if you want a title)", false)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        WelcomerProfile profile = guild.getWelcomerProfile();
        OptionMapping titleOption = event.getOption("title");
        OptionMapping descriptionOption = event.getOption("description");
        OptionMapping colorOption = event.getOption("color");
        OptionMapping pingBeforeSendOption = event.getOption("ping-before-send");
        if (descriptionOption == null || colorOption == null || pingBeforeSendOption == null) {
            return;
        }
        String title = titleOption == null ? null : titleOption.getAsString();
        String description = descriptionOption.getAsString();
        String color = colorOption.getAsString();
        boolean pingBeforeSend = pingBeforeSendOption.getAsBoolean();

        // Add the # if it's not there
        color = !color.startsWith("#") ? "#" + color : color;

        // Validate the input
        if (!HexColorUtils.isHexColor(color)){
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The color must be a valid hex color code Example: `#3498DB`\n" +
                            "You can use this website to get a hex color code: https://htmlcolorcodes.com")
                    .build()).queue();
            return;
        }
        if (title != null && title.length() > 128) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The title must be less than 128 characters")
                    .build()).queue();
            return;
        }
        if (description.length() > 512) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The description must be less than 512 characters")
                    .build()).queue();
            return;
        }

        boolean isMessageEnabled = profile.isMessage();
        profile.setEmbed(title, description, color, pingBeforeSend);
        DescriptionBuilder successDescription = new DescriptionBuilder("Welcomer Embed")
                .appendLine("%s Successfully set the welcomer embed!".formatted(Emojis.CHECK_MARK_EMOJI), false);
        if (isMessageEnabled) {
            successDescription.appendLine("*This has removed the plain message welcomer*", false);
        }

        successDescription.emptyLine();
        successDescription.appendLine("**Configuration:**", false);
        if (title != null) {
            successDescription.appendLine("Title: `%s`".formatted(title), true);
        }
        successDescription.appendLine("Description: `%s`".formatted(description), true);
        successDescription.appendLine("Color: `%s`".formatted(color), true);
        successDescription.appendLine("Ping Before Send: %s".formatted(pingBeforeSend ? Emojis.CHECK_MARK_EMOJI.getFormatted() + " *(Preview won't ping you)*" : Emojis.CROSS_MARK_EMOJI), true);
        successDescription.emptyLine();
        successDescription.appendLine("**Preview Below:**", false);
        event.replyEmbeds(
                EmbedUtils.successEmbed()
                        .setDescription(successDescription.build())
                        .build(),
                profile.getWelcomerEmbed().buildEmbed(guild, user).build()
        ).queue();
    }
}
