package cc.fascinated.bat.service;

import cc.fascinated.bat.common.NumberFormatter;
import cc.fascinated.bat.common.TimerUtils;
import cc.fascinated.bat.config.Config;
import cc.fascinated.bat.event.EventListener;
import lombok.Getter;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.entities.Activity;
import net.dv8tion.jda.api.requests.GatewayIntent;
import net.dv8tion.jda.api.utils.cache.CacheFlag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import java.util.EnumSet;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Service
@Getter
@Log4j2(topic = "Discord Service")
public class DiscordService implements EventListener {
    /**
     * The JDA instance
     */
    public static JDA JDA;

    private final List<String> messages = List.of(
            "/help for help",
            "/features to toggle features",
            "dsc.gg/batbot"
    );

    @Autowired
    public DiscordService(
            @NonNull ApplicationContext context,
            @Value("${discord.token}") String token
    ) throws Exception {
        context.getBean(Config.class); // Ensure the config is loaded
        log.info("Starting Discord bot...");
        JDA = JDABuilder.create(token, EnumSet.of(
                        GatewayIntent.GUILD_MESSAGES,
                        GatewayIntent.MESSAGE_CONTENT,
                        GatewayIntent.GUILD_MEMBERS,
                        GatewayIntent.GUILD_EMOJIS_AND_STICKERS,
                        GatewayIntent.GUILD_VOICE_STATES,
                        GatewayIntent.GUILD_INVITES
                ))
                .disableCache(
                        CacheFlag.ACTIVITY,
                        CacheFlag.CLIENT_STATUS,
                        CacheFlag.SCHEDULED_EVENTS,
                        CacheFlag.ONLINE_STATUS
                ).build()
                .awaitReady();
        log.info("Connected to Discord as {}", JDA.getSelfUser().getEffectiveName());
        TimerUtils.scheduleRepeating(this::updateActivity, 0, 1000 * 60 * 2);
    }

    /**
     * Updates the activity of the bot
     */
    public void updateActivity() {
        int guildCount = JDA.getGuilds().size();
        JDA.getPresence().setActivity(Activity.watching(messages.get(guildCount % messages.size())
                .replace("{guilds}", NumberFormatter.format(guildCount))
                .replace("{users}", NumberFormatter.format(JDA.getUsers().size()))
        ));
    }

    @Override
    public void onShutdown() {
        JDA.getPresence().setActivity(Activity.watching("Restarting..."));
    }
}
